@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright © 2022 Contrast Security, Inc.
+# Copyright © 2023 Contrast Security, Inc.
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
 import re
 
@@ -19,7 +19,7 @@ def cs__apply_trigger(
     if rule.disabled:
         return
 
-    source = rule.extract_source(node, orig_source)
+    source = rule.extract_source(node, orig_source, args, kwargs)
 
     if not node.dataflow_rule:
         if node.good_value:
