@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright © 2022 Contrast Security, Inc.
+# Copyright © 2023 Contrast Security, Inc.
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
 from contrast.agent.assess.policy.propagators import SUPPORTED_TYPES, BasePropagator
 from contrast.utils.assess.tag_utils import merge_tags
@@ -10,6 +10,7 @@ from contrast.agent.assess.utils import (
     is_tracked,
 )
 from contrast.utils.decorators import cached_property
+from contrast.utils.string_utils import ensure_string
 
 
 class JoinPropagator(BasePropagator):
@@ -68,7 +69,8 @@ class JoinPropagator(BasePropagator):
                 self._add_tags(self.target_properties, item_properties, target_offset)
                 copy_events(self.target_properties, item_properties)
 
-            target_offset += len(item)
+            # accounts for path-like objects, which have no __len__
+            target_offset += len(ensure_string(item))
 
             if i < len(self.strings_to_join) - 1 and base_properties is not None:
                 self._add_tags(self.target_properties, base_properties, target_offset)
