@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright © 2022 Contrast Security, Inc.
+# Copyright © 2023 Contrast Security, Inc.
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
 """
 Propagator used for flask.helpers.safe_join
@@ -12,6 +12,7 @@ from os import path
 from contrast.agent.assess.adjusted_span import AdjustedSpan
 from contrast.agent.assess.policy.propagators import JoinPropagator
 from contrast.utils.decorators import cached_property
+from contrast.utils.string_utils import ensure_string
 
 
 class SafeJoinPropagator(JoinPropagator):
@@ -60,10 +61,12 @@ class SafeJoinPropagator(JoinPropagator):
         In some cases the directory has a relative path `..` so we look for the
         absolute path.
         """
+        directory_str = ensure_string(self.directory)  # accounts for PosixPath
+
         directory = (
-            path.abspath(self.directory)
-            if self.use_abspath and "/.." in self.directory or "/." in self.directory
-            else self.directory
+            path.abspath(directory_str)
+            if self.use_abspath and "/.." in directory_str or "/." in directory_str
+            else directory_str
         )
 
         tag_span = AdjustedSpan(len(directory), len(self.target))
