@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright © 2022 Contrast Security, Inc.
+# Copyright © 2023 Contrast Security, Inc.
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
 from copy import copy
 
@@ -22,11 +22,12 @@ class DefaultAction(object):
     determine whether the rule was violated. These nodes can specify custom actions
     that implement the necessary logic.
 
-    Custum actions must implement (or inherit) the following three methods:
+    Custom actions must implement (or inherit) the following four attributes:
         * is_violated
         * find_all_tag_ranges
         * find_any_tag_ranges
         * extract_source
+        * unwind_source (boolean)
     """
 
     def is_violated(self, source_properties, required_tags, disallowed_tags, **kwargs):
@@ -97,10 +98,16 @@ class DefaultAction(object):
 
         return tag_ranges
 
-    def extract_source(self, source):
+    def extract_source(self, source, args, kwargs):
         """
         For some rules (eg ssrf) the source is an object containing a tracked string.
         By default, the source is unmodified.
         This method should be overridden as required.
         """
         return source
+
+    # for sources that are more complex data structures, like lists, dicts, etc., we
+    # call `cs__apply_source` once per element by default. Set this to False in a
+    # subclass to disable this behavior. This will cause `cs__apply_source` (and
+    # `extract_source`) to receive the original, unmodified source object.
+    unwind_source = True
