@@ -1,13 +1,16 @@
 # -*- coding: utf-8 -*-
-# Copyright © 2022 Contrast Security, Inc.
+# Copyright © 2023 Contrast Security, Inc.
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
 import re
 
 from contrast.agent.policy import constants
 from contrast.agent.assess.assess_exceptions import ContrastAssessException
 from contrast.agent.assess.policy.trigger_actions.default_action import DefaultAction
-from contrast.agent.assess.policy.trigger_actions import redos_action
-from contrast.agent.assess.policy.trigger_actions import ssrf_action
+from contrast.agent.assess.policy.trigger_actions import (
+    redos_action,
+    ssrf_action,
+    subprocess_action,
+)
 from contrast.agent.policy.policy_node import PolicyNode
 
 
@@ -17,6 +20,7 @@ class TriggerNode(PolicyNode):
         constants.TRIGGER_ACTION_DEFAULT: DefaultAction(),
         constants.TRIGGER_ACTION_REDOS: redos_action.RedosAction(),
         constants.TRIGGER_ACTION_SSRF: ssrf_action.SsrfAction(),
+        constants.TRIGGER_ACTION_SUBPROCESS: subprocess_action.SubprocessAction(),
     }
 
     def __init__(
