@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright © 2022 Contrast Security, Inc.
+# Copyright © 2023 Contrast Security, Inc.
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
 import os
 
@@ -117,8 +117,15 @@ class PathTraversalSinkFeatures(object):
 
     @staticmethod
     def _contains_known_attack_signatures(path):
-        unquoted = unquote(path)
-        unescaped = unquoted.encode("utf-8").decode("unicode-escape")
+        unescaped = unquoted = unquote(path)
+
+        try:
+            unescaped = unquoted.encode("utf-8").decode("unicode-escape")
+        except LookupError as e:
+            logger.debug(
+                f"Failed to unescape attack vector path {path} with the error {e}"
+            )
+            return False
 
         try:
             realpath = os.path.realpath(unescaped).lower().rstrip("/")
