@@ -1,9 +1,27 @@
 # -*- coding: utf-8 -*-
-# Copyright © 2022 Contrast Security, Inc.
+# Copyright © 2023 Contrast Security, Inc.
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
+import re
 from contrast.agent.protect.rule.deserialization.custom_searcher import CustomSearcher
 
 
+_MODULES = [
+    "os",
+    "system",
+    "sys",
+    "subprocess",
+    "__builtin__",
+    "builtins",
+    "globals",
+    "open",
+    "popen",
+]
+
+# we only need to know if the command begins with a module we care about (or "c" and
+# then the module)
+MODULE_REGEXES = [re.compile(r"^c?{}".format(module)) for module in _MODULES]
+
+
 class PickleSearcher(CustomSearcher):
     ID = "UD-PICKLE-1"
 
@@ -23,7 +41,9 @@ class PickleSearcher(CustomSearcher):
         count = 0
 
         for command in stack_commands:
-            contains_command = any(m in command for m in self.MODULES)
+            contains_command = any(
+                re.search(regex, command) for regex in MODULE_REGEXES
+            )
 
             if contains_command:
                 count += 1
@@ -38,15 +58,3 @@ class PickleSearcher(CustomSearcher):
             impact = self.IMPACT_HIGH
 
         return impact
-
-    MODULES = [
-        "os",
-        "system",
-        "sys",
-        "subprocess",
-        "__builtin__",
-        "builtins",
-        "globals",
-        "open",
-        "popen",
-    ]
