@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright © 2022 Contrast Security, Inc.
+# Copyright © 2023 Contrast Security, Inc.
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
 import os
 import socket
@@ -161,6 +161,12 @@ class _ServiceState(Singleton):
 
     def send_messages_retry(self, messages, attempts=0, wait=0, raise_exception=True):
         """
+        ****************
+        When using a nonzero value for `wait`, this function will cause the main python
+        process to sleep for an exponentially increasing amount of time on each retry
+        attempt.
+        ****************
+
         Attempt to send a list of messages to Speedracer. If sending messages fails, this method
         recursively calls itself to attempt to resend 3 times.
 
@@ -190,7 +196,13 @@ class _ServiceState(Singleton):
         :return: list of responses from Speedracer
         :side effect: ContrastServiceException if raise_exception is True
         """
-        service_util.sleep(wait)
+        if wait > 0:
+            logger.info(
+                "Waiting %s second%s for Contrast Service",
+                wait,
+                "" if wait == 1 else "s",
+            )
+            service_util.sleep(wait)
 
         try:
             return self._send_messages(messages)
@@ -200,18 +212,19 @@ class _ServiceState(Singleton):
             )
 
             if attempts < 3:
-                if self.bundled_flag == "bundled":
-                    # this may lead to multiple contrast-service processes existing on the same host if
-                    # the first initialized process is not up before other app workers / threads attempt
-                    # to start the service.
-                    self.start_service()
-                logger.debug("Retrying to send messages...")
+                new_attempts = attempts + 1
+                new_wait = wait * 2
+                logger.debug(
+                    "Retrying to send messages: retry attempt #%s, wait=%s",
+                    new_attempts,
+                    new_wait,
+                )
                 return self.send_messages_retry(
-                    messages, attempts + 1, wait, raise_exception
+                    messages, new_attempts, new_wait, raise_exception
                 )
 
             logger.warning(
-                "Unable to send messages to Contrast Service after 3 attemps."
+                "Unable to send messages to Contrast Service after 3 retries."
             )
             self.connection_status.failure()
             if raise_exception:
