@@ -1,7 +1,5 @@
 # coding: utf-8
 
-from __future__ import print_function, absolute_import, division, unicode_literals
-
 """
 You cannot subclass bool, and this is necessary for round-tripping anchored
 bool values (and also if you want to preserve the original way of writing)
@@ -18,14 +16,12 @@ if False:  # MYPY
 
 __all__ = ['ScalarBoolean']
 
-# no need for no_limit_int -> int
-
 
 class ScalarBoolean(int):
     def __new__(cls, *args, **kw):
         # type: (Any, Any, Any) -> Any
-        anchor = kw.pop('anchor', None)  # type: ignore
-        b = int.__new__(cls, *args, **kw)  # type: ignore
+        anchor = kw.pop('anchor', None)
+        b = int.__new__(cls, *args, **kw)
         if anchor is not None:
             b.yaml_set_anchor(anchor, always_dump=True)
         return b
