@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright © 2022 Contrast Security, Inc.
+# Copyright © 2023 Contrast Security, Inc.
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
 """
 Implementation of patches for the `re` module.
@@ -34,7 +34,12 @@ def build_group_hook(original_func, patch_policy, propagator):
     def group_hook(*args, **kwargs):
         context = contrast.CS__CONTEXT_TRACKER.current()
         result = original_func(*args, **kwargs)
-        if context is not None and context.propagate_assess:
+        if (
+            context is not None
+            and context.propagate_assess
+            and not context.stop_propagation
+        ):
+            context.propagated()
             with scope.propagation_scope():
                 propagator(args[0], result, *args[1:])
         return result
@@ -172,8 +177,9 @@ def build_sub_hook(original_func, policy_node):
         """
         # Get the non-propagation case out of the way here
         context = contrast.CS__CONTEXT_TRACKER.current()
-        if context is None or not context.propagate_assess:
+        if context is None or not context.propagate_assess or context.stop_propagation:
             return original_func(*args, **kwargs)
+        context.propagated()
 
         try:
             repl = args[1]
@@ -199,8 +205,9 @@ def build_sub_hook(original_func, policy_node):
 @fail_quietly("Failed to analyze")
 def _analyze_policy(name, result, args, kwargs):
     context = contrast.CS__CONTEXT_TRACKER.current()
-    if context is None or not context.propagate_assess:
+    if context is None or not context.propagate_assess or context.stop_propagation:
         return
+    context.propagated()
 
     with scope.contrast_scope():
         policy = Policy()
